// This file is the estimation file for "The Bullionist Controversy: A Bayesian DSGE Approach"
// by Joshua R. Hendrickson.

// Define the Variables

var qcr r phi br varrhocr varrhocd p m g e qcrstar rstar phistar brstar varrhocrstar varrhocdstar pstar mstar gstar emm egg;

varexo em eg emstar egstar;

// Define the Parameters

parameters alpha beta theta delta rhom rhog NI I gamma rhoem rhoeg rhomstar rhogstar;

alpha = 0.1;
beta = 0.99;
theta = 0.9;
delta = 2;
rhom = 0.86;
rhog = 0.53;
NI = 228;
I = 1.05;
gamma = 0;
rhoem = 0.5;
rhoeg = 0.5;
rhomstar = 0.5;
rhogstar = 0.5;


// Define the Model

model;
phi/(beta*phi(+1)) = alpha*theta*(varrhocr(+1)^(-delta) - varrhocr(+1)) + 1;
varrhocd + (2/NI)*e*gstar = 1;
varrhocr + (2/NI)*e*gstar = qcr;
br = qcr - ((1-theta)*(varrhocr^(1-delta))/(1-delta) + theta*(1/2)*varrhocr^2) - (4/NI)*gstar;
phi*m = (NI/4)*((1-theta)*(varrhocr^(1-delta))/(1-delta) + theta*(1/2)*varrhocr^2);
p = 1/phi;
g = phi*(m - m(-1)) + (r-1)*(NI/4)*br;
ln(m) = rhom*ln(m(-1)) + (1-rhom)*gamma*ln(g) + emm;
ln(g) = rhog*ln(g(-1)) + egg;
emm = rhoem*emm(-1) + em;
egg = rhoeg*egg(-1) + eg;
e = phi/phistar;
phistar/(beta*phistar(+1)) = alpha*theta*(varrhocrstar(+1)^(-delta) - varrhocrstar(+1)) + 1;
varrhocdstar + (2/NI)*(1/e)*g = 1;
varrhocrstar + (2/NI)*(1/e)*g = qcrstar;
brstar = qcrstar - ((1-theta)*(varrhocrstar^(1-delta))/(1-delta) + theta*(1/2)*varrhocrstar^2) - (4/NI)*g;
phistar*mstar = (NI/4)*((1-theta)*(varrhocrstar^(1-delta))/(1-delta) + theta*(1/2)*varrhocrstar^2);
pstar = 1/phistar;
gstar = phistar*(mstar - mstar(-1)) + (rstar-1)*(NI/4)*brstar;
ln(mstar) = rhomstar*ln(mstar(-1)) + emstar;
ln(gstar) = rhogstar*ln(gstar(-1)) + egstar;
end;

// Define the Steady State

initval;
m = 1;
g = 1;
mstar = 1;
gstar = 1;
r = 1.05;
qcr = 1/r + gstar;
varrhocd = 1 - (1/NI)*gstar;
varrhocr = (((1/beta - 1)/(alpha*theta)) + (1/I))^(-1/delta);
phi = (NI/4)*((1-theta)*(varrhocr^(1-delta))/(1-delta) + theta*(1/2)*varrhocr^2);
p = 1/phi;
br = qcr - ((1-theta)*(varrhocr^(1-delta))/(1-delta) + theta*(1/2)*varrhocr^2) - (1/NI)*g;
rstar = 1.05;
qcrstar = 1/rstar + g;
varrhocdstar = 1 - (1/NI)*g;
varrhocrstar = (((1/beta - 1)/(alpha*theta)) + (1/I))^(-1/delta);
phistar = (NI/4)*((1-theta)*(varrhocrstar^(1-delta))/(1-delta) + theta*(1/2)*varrhocrstar^2);
pstar = 1/phistar;
brstar = qcrstar - ((1-theta)*(varrhocrstar^(1-delta))/(1-delta) + theta*(1/2)*varrhocrstar^2) - (1/NI)*gstar;
e = phi/phistar;
end;
steady;

estimated_params;
//beta,beta_pdf,0.99,0.001;
delta,gamma_pdf,2,0.5;
rhom,beta_pdf,0.5,0.1;
rhog,beta_pdf,0.5,0.1;
rhoem,beta_pdf,0.5,0.1;
rhoeg,beta_pdf,0.5,0.1;
rhomstar,beta_pdf,0.5,0.2;
rhogstar,beta_pdf,0.5,0.2;
gamma,uniform_pdf,,,-2,2;
stderr em,inv_gamma_pdf,0.01,inf;
stderr eg,inv_gamma_pdf,0.01,inf;
stderr egstar,inv_gamma_pdf,0.01,inf;
stderr emstar,inv_gamma_pdf,0.01,inf;
stderr g,uniform_pdf,,,0,10;
end;

varobs m g p e;

estimation(datafile=bdata,mh_jscale=0.225,mode_compute=6,bayesian_irf,moments_varendo,mh_replic=250000,mh_drop=0.2);